/* vim: set textwidth=80 tabstop=4: */

/*
 * Claws Mail -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2008 Michael Rasmussen and the Claws Mail Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#include "defs.h"

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>

#include "gtk/gtkutils.h"
#include "common/claws.h"
#include "common/version.h"
#include "plugin.h"
#include "mainwindow.h"
#include "utils.h"
#include "prefs.h"
#include "folder.h"
#include "foldersel.h"
#include "procmsg.h"
#include "libarchive_archive.h"
#include "archiver.h"

static struct ArchivePage page;

static void init_archive_page() {
	page.folder = NULL;
	page.name = NULL;
	page.response = FALSE;
}

static void dispose_archive_page() {
	if (page.folder)
		g_free(page.folder);
	if (page.name)
		g_free(page.name);
}

static void archiver_save_files(gchar* folder, gchar* archive) {
	FolderItem* item = folder_find_item_from_identifier(folder);
	
	GSList *msglist = folder_item_get_msg_list(item);
	GSList *cur;
	for (cur = msglist; cur; cur = cur->next) {
		gchar *file = procmsg_get_message_file(cur->data);
		/*debug_print("Processing: %s\n", file);*/
		archive_add_file(file);
	}
	procmsg_msg_list_free(msglist);
	GSList* list = archive_get_file_list();
	if (archive != NULL)
		archive_create(archive, list);
	archive_free_file_list();
}

static void entry_change_cb(GtkWidget* widget, gpointer data) {
	const gchar* name = gtk_widget_get_name(widget);

	if (strcmp("folder", name) == 0) {
		page.folder = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
		debug_print("page.folder = %s\n", page.folder);
	}
	else if (strcmp("file", name) == 0) {
		page.name = g_strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
		debug_print("page.name = %s\n", page.name);
	}
}

static void archiver_dialog_cb(GtkWidget* widget, gint action, gpointer data) {
	switch (action) {
		case GTK_RESPONSE_ACCEPT:
			debug_print("User chose OK\n");
			page.response = TRUE;
			break;
		default:
			debug_print("User chose CANCEL\n");
			page.response = FALSE;
	}
	gtk_widget_destroy(widget);
	debug_print("Settings:\nfolder: %s\nname: %s\naction: %d\n",
			page.folder, page.name, page.response);
	archiver_save_files(page.folder, page.name);
	dispose_archive_page();
}

static void foldersel_cb(GtkWidget *widget, gpointer data)
{
	FolderItem *item;
	gchar *item_id;
	gint newpos = 0;

	item = foldersel_folder_sel(NULL, FOLDER_SEL_MOVE, NULL, FALSE);
	if (item && (item_id = folder_item_get_identifier(item)) != NULL) {
		gtk_editable_delete_text(GTK_EDITABLE(data), 0, -1);
		gtk_editable_insert_text(GTK_EDITABLE(data), item_id, strlen(item_id), &newpos);
		page.folder = g_strdup(item_id);
		g_free(item_id);
	}
	debug_print("Folder to archive: %s\n", 
				gtk_entry_get_text(GTK_ENTRY(data)));
}

static void filesel_cb(GtkWidget *widget, gpointer data)
{
	GtkWidget *dialog;
	gchar* file;
	gint newpos = 0;
	const gchar* homedir;

	dialog = gtk_file_chooser_dialog_new(
			"Select file name for archive [suffix should be .tar.gz or .tgz]",
			NULL,
			GTK_FILE_CHOOSER_ACTION_SAVE,
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			GTK_STOCK_APPLY, GTK_RESPONSE_APPLY,
			NULL);
	homedir = g_getenv("HOME");
	if (!homedir)
		homedir = g_get_home_dir();

	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), homedir);
	if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_APPLY) {
		file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
		if (file) {
			gtk_editable_delete_text(GTK_EDITABLE(data), 0, -1);
			gtk_editable_insert_text(GTK_EDITABLE(data), file, strlen(file), &newpos);
			page.name = g_strdup(file);
			g_free(file);
		}
	}
	gtk_widget_destroy(dialog);
	debug_print("Name for archive: %s\n",
				gtk_entry_get_text(GTK_ENTRY(data)));
}

void archiver_gtk_show() {
	GtkWidget* dialog;
	GtkWidget* frame;
	GtkWidget* vbox1;
	GtkWidget* hbox1;
	GtkWidget* folder;
	GtkWidget* folder_label;
	GtkWidget* folder_select;
	GtkWidget* file;
	GtkWidget* file_label;
	GtkWidget* file_select;
	GtkTooltips* tooltips;

	MainWindow* mainwin = mainwindow_get_mainwindow();

	init_archive_page();

	tooltips = gtk_tooltips_new();

	dialog = gtk_dialog_new_with_buttons (
				_("Mail Archiver Dialog"),
				GTK_WINDOW(mainwin->window),
				GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_STOCK_CANCEL,
				GTK_RESPONSE_CANCEL,
				GTK_STOCK_OK,
				GTK_RESPONSE_ACCEPT,
				NULL);

	g_signal_connect (
				dialog,
				"response",
				G_CALLBACK(archiver_dialog_cb),
				NULL);

	frame = gtk_frame_new("Enter Archiver arguments");
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_OUT);
	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);

	vbox1 = gtk_vbox_new (FALSE, 4);
	gtk_widget_show (vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);
	gtk_container_add(GTK_CONTAINER(frame), vbox1);
	
	hbox1 = gtk_hbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 8);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	folder_label = gtk_label_new(_("Folder to archive"));
	gtk_box_pack_start(GTK_BOX(hbox1), folder_label, FALSE, FALSE, 0);
	
	folder = gtk_entry_new();
	gtk_widget_set_name(folder, "folder");
	gtk_box_pack_start(GTK_BOX(hbox1), folder, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tooltips, folder,
			_("Folder which is root of archive"), NULL);

	folder_select = gtkut_get_browse_directory_btn(_("_Browse"));
	gtk_box_pack_start(GTK_BOX(hbox1), folder_select, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, folder_select,
			_("Click this button to select a folder which is to be root of archive"), NULL);

	hbox1 = gtk_hbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(hbox1), 8);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);

	file_label = gtk_label_new(_("Name for archive"));
	gtk_box_pack_start(GTK_BOX(hbox1), file_label, FALSE, FALSE, 0);
	
	file = gtk_entry_new();
	gtk_widget_set_name(file, "file");
	gtk_box_pack_start(GTK_BOX(hbox1), file, TRUE, TRUE, 0);
	gtk_tooltips_set_tip(tooltips, file,
			_("Archive name"), NULL);

	file_select = gtkut_get_browse_directory_btn(_("_Select"));
	gtk_box_pack_start(GTK_BOX(hbox1), file_select, FALSE, FALSE, 0);
	gtk_tooltips_set_tip(tooltips, file_select,
			_("Click this button to select a name for your archive"), NULL);

	g_signal_connect(G_OBJECT(folder_select), "clicked", 
			 G_CALLBACK(foldersel_cb), folder);
	g_signal_connect(G_OBJECT(file_select), "clicked",
			 G_CALLBACK(filesel_cb), file);
	g_signal_connect(G_OBJECT(folder), "activate",
			 G_CALLBACK(entry_change_cb), NULL);
	g_signal_connect(G_OBJECT(file), "activate",
			 G_CALLBACK(entry_change_cb), NULL);

	gtk_widget_show_all(dialog);
}

void archiver_gtk_done() {
}

